import urllib.parse
from ecotricity.client.dto import AgreementsResult, Session
from ecotricity.client.requests.BaseRequest import BaseRequest


class AgreementsRequest(BaseRequest):
    path: str

    def __init__(self, host="api.ecotricity.co.uk", proto="https", path="/customers/v1/"):
        super().__init__(host, proto)
        self.path = path

    def get_agreements(self, session: Session, account_id: str,) -> AgreementsResult:
        variables = urllib.parse.quote(f'customers/{session.customer_id}/accounts/{account_id}/agreements')

        return self.read_data(self.path, variables, session, AgreementsResult)



from dataclasses import dataclass
from dataclass_wizard import JSONWizard


@dataclass
class BillingAddress(JSONWizard):
    postcode: str
    country_code: str
    address1: str = None
    address2: str = None
    address3: str = None
    address4: str = None



from dataclasses import dataclass
from datetime import date
from dataclass_wizard import JSONWizard
from ecotricity.client.dto.Product import Product


@dataclass
class Agreement(JSONWizard):
    id: str
    display_number: str
    from_date: date
    fuel_type: str
    products: list[Product]



from json import JSONDecodeError
from urllib.error import HTTPError
from urllib.request import Request, urlopen
from dataclass_wizard import JSONWizard
from ecotricity.client.dto import Session
from ecotricity.client.exceptions import ApiException, ResponseDecodeException, EcotricityClientException


class BaseRequest:
    host: str
    proto: str

    def __init__(self, host="api.ecotricity.co.uk", proto="https"):
        self.host = host
        self.proto = proto

    def read_data(self, path: str, variables: str, session: Session, target: type[JSONWizard]):
        r = Request(f'{self.proto}://{self.host}{path}{variables}')
        r.add_header('Authorization', f'Bearer {session.auth_token}')

        try:
            return target.from_json(urlopen(r).read().decode())
        except HTTPError as ex:
            raise ApiException("Failed to read from the API") from ex
        except JSONDecodeError as ex:
            raise ResponseDecodeException("Unable to decode response") from ex
        except Exception as ex:
            raise EcotricityClientException("Error while getting readings") from ex



from dataclasses import dataclass
from datetime import datetime
from dataclass_wizard import JSONWizard


@dataclass
class MeterRead(JSONWizard):
    id: str
    date: datetime
    created_at: datetime
    updated_at: datetime
    source: str
    value: float
    quality: str
    sequence_type: str
    register_industry_id: str
    meter_industry_id: str
    status: str
    submission_client: str
    account_id: str
    agreement_id: str
    meter_point_industry_id: str



import urllib.parse
from ecotricity.client.dto import AccountsResult, Session
from ecotricity.client.requests.BaseRequest import BaseRequest


class AccountsRequest(BaseRequest):
    path: str

    def __init__(self, host="api.ecotricity.co.uk", proto="https", path="/customers/v1/"):
        super().__init__(host, proto)
        self.path = path

    def get_accounts(self, session: Session) -> AccountsResult:
        variables = urllib.parse.quote(f'customers/{session.customer_id}/accounts')

        return self.read_data(self.path, variables, session, AccountsResult)



import json
from json import JSONDecodeError
from urllib.error import HTTPError
from urllib.request import Request, urlopen
from ecotricity.client.dto import TokensResult
from ecotricity.client.exceptions import ApiException
from ecotricity.client.exceptions import EcotricityClientException
from ecotricity.client.exceptions import ResponseDecodeException
from ecotricity.client.requests.BaseRequest import BaseRequest


class TokensRequest(BaseRequest):
    path: str

    def __init__(self, host="api.ecotricity.co.uk", proto="https", path="/auth/v2/tokens"):
        super().__init__(host, proto)
        self.path = path

    def get_tokens(self, username: str, password: str) -> TokensResult:
        body = str(json.dumps({'username': username, 'password': password})).encode('utf-8')
        r = Request(f'{self.proto}://{self.host}{self.path}')

        try:
            return TokensResult.from_json(urlopen(r, data=body).read().decode())
        except HTTPError as ex:
            raise ApiException("Failed to read from the API") from ex
        except JSONDecodeError as ex:
            raise ResponseDecodeException("Unable to decode response") from ex
        except Exception as ex:
            raise EcotricityClientException("Error while getting readings") from ex



from .Account import Account
from .AccountsData import AccountsData
from .AccountsResult import AccountsResult
from .Agreement import Agreement
from .AgreementsData import AgreementsData
from .AgreementsResult import AgreementsResult
from .BillingAddress import BillingAddress
from .MeterPoint import MeterPoint
from .MeterRead import MeterRead
from .MeterReadsData import MeterReadsData
from .MeterReadsResult import MeterReadsResult
from .Product import Product
from .Session import Session
from .TokensData import TokensData
from .TokensResult import TokensResult



import urllib.parse
from ecotricity.client.dto import MeterReadsResult, Session
from ecotricity.client.requests.BaseRequest import BaseRequest


class ReadsRequest(BaseRequest):
    path: str

    def __init__(self, host="api.ecotricity.co.uk", proto="https", path="/meters/v1/"):
        super().__init__(host, proto)
        self.path = path

    def get_reads(self, session: Session, meter_point: str) -> MeterReadsResult:
        variables = urllib.parse.quote(f'customers/{session.customer_id}/meter-points/{meter_point}/reads')

        return self.read_data(self.path, variables, session, MeterReadsResult)



import pathlib

from setuptools import setup

here = pathlib.Path(__file__).parent.resolve()

VERSION = "1.0.1"
LONG_DESCRIPTION = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="ecotricity-client",
    version=VERSION,
    author="Derek Kaye",
    description="Wrapper around the Ecotricity API",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://github.com/dezzak/ecotricity-client"
)



from .AccountsRequest import AccountsRequest
from .AgreementsRequest import AgreementsRequest
from .BaseRequest import BaseRequest
from .ReadsRequest import ReadsRequest
from .TokensRequest import TokensRequest



import datetime
from dataclasses import dataclass
from dataclass_wizard import JSONWizard

from ecotricity.client.dto.BillingAddress import BillingAddress


@dataclass
class Account(JSONWizard):
    id: str
    from_date: datetime.date
    balance: float
    display_number: str
    billing_method: str
    display_billing_address: BillingAddress
    payment_method: str
    account_class: str
    in_dunning: bool



import jwt

from ecotricity.client.dto import Session
from ecotricity.client.requests import TokensRequest


def open_session(username: str, password: str) -> Session:
    request = TokensRequest()
    result = request.get_tokens(username, password)
    auth_token = result.data.access
    claims = jwt.decode(auth_token, options={'verify_signature': False})
    customer_id = claims['data']['customerId']
    return Session(customer_id, auth_token)



from dataclasses import dataclass
from dataclass_wizard import JSONWizard
from ecotricity.client.dto.MeterPoint import MeterPoint


@dataclass
class Product(JSONWizard):
    display_name: str
    meter_points: list[MeterPoint]
